#include "gmozillaprompter.h"

enum {
    INCLUDE_USERNAME = 1 << 0,
    INCLUDE_PASSWORD = 1 << 1,
    INCLUDE_CHECKBOX = 1 << 2,
    INCLUDE_CANCEL   = 1 << 3
};

struct DialogDescription {
    int    flags;
    gchar* icon;
};


static const DialogDescription DialogTable[] = {
    { 0,                      "qgn_note_gene_syswarning"  },  // ALERT
    { INCLUDE_CHECKBOX,       "qgn_note_browser_java_alert"  },  // ALERT_CHECK
    { INCLUDE_CANCEL,         "qgn_note_confirm" },  // CONFIRM
    { INCLUDE_CHECKBOX |
      INCLUDE_CANCEL,         "qgn_note_confirm" },  // CONFIRM_CHECK
    { INCLUDE_CANCEL |
      INCLUDE_CHECKBOX,       "qgn_note_browser_java_prompt" },  // PROMPT
    { INCLUDE_CANCEL |
      INCLUDE_USERNAME |
      INCLUDE_PASSWORD |
      INCLUDE_CHECKBOX,       "qgn_note_browser_java_prompt" },  // PROMPT_USER_PASS
    { INCLUDE_CANCEL |
      INCLUDE_PASSWORD |
      INCLUDE_CHECKBOX,       "qgn_note_browser_java_prompt"  },  // PROMPT_PASS
    { INCLUDE_CANCEL,         "qgn_note_confirm"  },  // SELECT
    { INCLUDE_CANCEL |
      INCLUDE_CHECKBOX,       "qgn_note_confirm" }   // UNIVERSAL
};

MozillaPrompter::MozillaPrompter(void)
    : mCheckValue(PR_FALSE),
      mItemList(nsnull),
      mItemCount(0),
      mButtonPressed(0),
      mConfirmResult(PR_FALSE),
      mSelectedItem(0),
      mWindow(NULL),
      mUserField(NULL),
      mPassField(NULL),
      mTextField(NULL),
      mOptionMenu(NULL),
      mCheckBox(NULL)
{
}

MozillaPrompter::~MozillaPrompter(void)
{
    if (mItemList)
        delete[] mItemList;
}

nsresult MozillaPrompter::Create(PromptType aType, GtkWindow* aParentWindow)
{
    mWindow = gtk_dialog_new ();
    
    gtk_window_set_title (GTK_WINDOW(mWindow),mTitle.get());
    //gtk_window_set_decorated (GTK_WINDOW(mWindow),FALSE);
    
    if (aParentWindow && aParentWindow->group) {
        gtk_window_group_add_window (aParentWindow->group, GTK_WINDOW (mWindow));
    }

    GtkWidget* dialogHBox = gtk_hbox_new(FALSE, 12);

    gtk_container_set_border_width(GTK_CONTAINER(mWindow), 6);
    gtk_dialog_set_has_separator(GTK_DIALOG(mWindow), FALSE);
    gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(mWindow)->vbox), 12);
    gtk_container_set_border_width(GTK_CONTAINER(dialogHBox), 6);

    GtkWidget* contentsVBox = gtk_vbox_new(FALSE, 12);

    const gchar* iconDesc = DialogTable[aType].icon;
    GtkWidget* icon = gtk_image_new_from_icon_name(iconDesc, HILDON_ICON_SIZE_50); //GTK_ICON_SIZE_DIALOG
    gtk_misc_set_alignment(GTK_MISC(icon), 0.5, 0.0);
    gtk_box_pack_start(GTK_BOX(dialogHBox), icon, FALSE, FALSE, 0);

    GtkWidget* label = gtk_label_new(mMessageText.get());
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_label_set_selectable(GTK_LABEL(label), FALSE);
    gtk_widget_modify_font(label, pango_font_description_from_string("sans 14"));
    gtk_box_pack_start(GTK_BOX(contentsVBox), label, FALSE, FALSE, 0);

    int widgetFlags = DialogTable[aType].flags;

    if (widgetFlags & (INCLUDE_USERNAME | INCLUDE_PASSWORD)) {

        GtkWidget* userPassHBox = gtk_hbox_new(FALSE, 12);
        GtkWidget* userPassLabels = gtk_vbox_new(TRUE, 6);
        GtkWidget* userPassFields = gtk_vbox_new(TRUE, 6);

        if (widgetFlags & INCLUDE_USERNAME) {
            GtkWidget* userLabel = gtk_label_new("User Name:");
            gtk_box_pack_start(GTK_BOX(userPassLabels), userLabel, FALSE,
                               FALSE, 0);

            mUserField = gtk_entry_new();

            if (!mUser.IsEmpty())
                gtk_entry_set_text(GTK_ENTRY(mUserField), mUser.get());

            gtk_entry_set_activates_default(GTK_ENTRY(mUserField), TRUE);

            gtk_box_pack_start(GTK_BOX(userPassFields), mUserField, FALSE,
                               FALSE, 0);
        }
        if (widgetFlags & INCLUDE_PASSWORD) {
            GtkWidget* passLabel = gtk_label_new("Password:");
            gtk_box_pack_start(GTK_BOX(userPassLabels), passLabel, FALSE,
                               FALSE, 0);

            mPassField = gtk_entry_new();

            if (!mPass.IsEmpty())
                gtk_entry_set_text(GTK_ENTRY(mPassField), mPass.get());

            gtk_entry_set_visibility(GTK_ENTRY(mPassField), FALSE);
            gtk_entry_set_activates_default(GTK_ENTRY(mPassField), TRUE);

            gtk_box_pack_start(GTK_BOX(userPassFields), mPassField, FALSE,
                               FALSE, 0);
        }

        gtk_box_pack_start(GTK_BOX(userPassHBox), userPassLabels, FALSE,
                           FALSE, 0);
        gtk_box_pack_start(GTK_BOX(userPassHBox), userPassFields, FALSE,
                           FALSE, 0);
        gtk_box_pack_start(GTK_BOX(contentsVBox), userPassHBox, FALSE, FALSE, 0);
    }

    if (aType == TYPE_PROMPT) {
        mTextField = gtk_entry_new();

        if (!mTextValue.IsEmpty())
            gtk_entry_set_text(GTK_ENTRY(mTextField), mTextValue.get());

        gtk_entry_set_activates_default(GTK_ENTRY(mTextField), TRUE);

        gtk_box_pack_start(GTK_BOX(contentsVBox), mTextField, FALSE, FALSE, 0);
    }

    if ((widgetFlags & INCLUDE_CHECKBOX) && !mCheckMessage.IsEmpty()) {

        GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
        gtk_box_pack_start(GTK_BOX(contentsVBox), hbox, FALSE, FALSE, 0);
        
        mCheckBox = gtk_check_button_new();
        gtk_box_pack_start(GTK_BOX(hbox), mCheckBox, FALSE, FALSE, 0);
        GtkWidget *label2 = gtk_label_new(mCheckMessage.get());
        gtk_box_pack_start(GTK_BOX(hbox), label2, FALSE, FALSE, 0);
        gtk_widget_modify_font(label2, pango_font_description_from_string("sans 12"));
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mCheckBox),
                                     mCheckValue);
        gtk_label_set_line_wrap(GTK_LABEL(gtk_bin_get_child(GTK_BIN(mCheckBox))),
                               TRUE);

        gtk_box_pack_start(GTK_BOX(contentsVBox), mCheckBox, FALSE, FALSE, 0);
    }

    if (aType == TYPE_SELECT) {
        GtkWidget* menu = gtk_menu_new();
        for (PRUint32 i = 0; i < mItemCount; ++i) {
            GtkWidget* item = gtk_menu_item_new_with_label(mItemList[i].get());
            gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
        }

        mOptionMenu = gtk_option_menu_new();

        gtk_option_menu_set_menu(GTK_OPTION_MENU(mOptionMenu), menu);
        gtk_box_pack_start(GTK_BOX(contentsVBox), mOptionMenu, FALSE, FALSE, 0);
    }

    if (aType == TYPE_UNIVERSAL) {
        for (int i = EMBED_MAX_BUTTONS; i >= 0; --i) {
            if (!mButtonLabels[i].IsEmpty())
                gtk_dialog_add_button(GTK_DIALOG(mWindow),
                                      mButtonLabels[i].get(), i);
        }
        gtk_dialog_set_default_response(GTK_DIALOG(mWindow), 0);
    } else {
        if (widgetFlags & INCLUDE_CANCEL)
            gtk_dialog_add_button(GTK_DIALOG(mWindow), GTK_STOCK_CANCEL,
                                  GTK_RESPONSE_CANCEL);

        GtkWidget* okButton = gtk_dialog_add_button(GTK_DIALOG(mWindow),
                                                    GTK_STOCK_OK,
                                                    GTK_RESPONSE_ACCEPT);
        gtk_widget_grab_default(okButton);
    }

    gtk_box_pack_start(GTK_BOX(dialogHBox), contentsVBox, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(mWindow)->vbox), dialogHBox, FALSE, FALSE, 0);
    return NS_OK;
}

void MozillaPrompter::SetTitle(const PRUnichar *aTitle)
{
    CopyUTF16toUTF8(aTitle, mTitle);
}

void MozillaPrompter::SetTextValue(const PRUnichar *aTextValue)
{
    CopyUTF16toUTF8(aTextValue, mTextValue);
}

void MozillaPrompter::SetCheckMessage(const PRUnichar *aMessage)
{
    CopyUTF16toUTF8(aMessage, mCheckMessage);
}

void MozillaPrompter::SetMessageText(const PRUnichar *aMessageText)
{
    CopyUTF16toUTF8(aMessageText, mMessageText);
}

void MozillaPrompter::SetUser(const PRUnichar *aUser)
{
    CopyUTF16toUTF8(aUser, mUser);
}

void MozillaPrompter::SetPassword(const PRUnichar *aPass)
{
    CopyUTF16toUTF8(aPass, mPass);
}

void MozillaPrompter::SetCheckValue(const PRBool aValue)
{
    mCheckValue = aValue;
}

void MozillaPrompter::SetItems(const PRUnichar** aItemArray, PRUint32 aCount)
{
    if (mItemList)
        delete[] mItemList;

    mItemCount = aCount;
    mItemList = new nsCString[aCount];
    for (PRUint32 i = 0; i < aCount; ++i)
        CopyUTF16toUTF8(aItemArray[i], mItemList[i]);
}

void MozillaPrompter::SetButtons(const PRUnichar* aButton0Label,
                                const PRUnichar* aButton1Label,
                                const PRUnichar* aButton2Label)
{
    CopyUTF16toUTF8(aButton0Label, mButtonLabels[0]);
    CopyUTF16toUTF8(aButton1Label, mButtonLabels[1]);
    CopyUTF16toUTF8(aButton2Label, mButtonLabels[2]);
}

void MozillaPrompter::GetCheckValue(PRBool *aValue)
{
    *aValue = mCheckValue;
}

void MozillaPrompter::GetConfirmValue(PRBool *aConfirmValue)
{
    *aConfirmValue = mConfirmResult;
}
 
void MozillaPrompter::GetTextValue(PRUnichar **aTextValue)
{
    *aTextValue = UTF8ToNewUnicode(mTextValue);
}

void MozillaPrompter::GetUser(PRUnichar **aUser)
{
    *aUser = UTF8ToNewUnicode(mUser);
}

void MozillaPrompter::GetPassword(PRUnichar **aPass)
{
    *aPass = UTF8ToNewUnicode(mPass);
}

void MozillaPrompter::GetSelectedItem(PRInt32 *aIndex)
{
    *aIndex = mSelectedItem;
}

void MozillaPrompter::GetButtonPressed(PRInt32 *aButton)
{
    *aButton = mButtonPressed;
}

void MozillaPrompter::Run(void)
{
    gtk_widget_show_all(mWindow);
    gint response = gtk_dialog_run(GTK_DIALOG(mWindow));
    switch (response) {
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_DELETE_EVENT:
        mConfirmResult = PR_FALSE;
        break;
    case GTK_RESPONSE_ACCEPT:
        mConfirmResult = PR_TRUE;
        SaveDialogValues();
        break;
    default:
        mButtonPressed = response;
        SaveDialogValues();
    }

    gtk_widget_destroy(mWindow);
}

void MozillaPrompter::SaveDialogValues()
{
    if (mUserField)
        mUser.Assign(gtk_entry_get_text(GTK_ENTRY(mUserField)));

    if (mPassField)
        mPass.Assign(gtk_entry_get_text(GTK_ENTRY(mPassField)));

    if (mCheckBox)
        mCheckValue = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mCheckBox));

    if (mTextField)
        mTextValue.Assign(gtk_entry_get_text(GTK_ENTRY(mTextField)));

    if (mOptionMenu)
        mSelectedItem = gtk_option_menu_get_history(GTK_OPTION_MENU(mOptionMenu));
}

