#include "gmozillapromptservice.h"
#include "gmozillaprompter.h"

PromptService::PromptService()
{
}

PromptService::~PromptService()
{
}

NS_IMPL_ISUPPORTS1(PromptService, nsIPromptService)

NS_IMETHODIMP PromptService::Alert(nsIDOMWindow* aParent, const PRUnichar* aDialogTitle,
                                    const PRUnichar* aDialogText)
{
    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Alert").get());
    prompter.SetMessageText(aDialogText);
    prompter.Create(MozillaPrompter::TYPE_ALERT, GetWindowForDOMWindow(aParent));
    prompter.Run();
    return NS_OK;
}

NS_IMETHODIMP PromptService::AlertCheck(nsIDOMWindow* aParent,
                                        const PRUnichar* aDialogTitle,
                                        const PRUnichar* aDialogText,
                                        const PRUnichar* aCheckMsg, PRBool* aCheckValue)
{
    NS_ENSURE_ARG_POINTER(aCheckValue);

    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Alert").get());
    prompter.SetMessageText(aDialogText);
    prompter.SetCheckMessage(aCheckMsg);
    prompter.SetCheckValue(*aCheckValue);
    prompter.Create(MozillaPrompter::TYPE_ALERT_CHECK, GetWindowForDOMWindow(aParent));
    prompter.Run();
    prompter.GetCheckValue(aCheckValue);
    return NS_OK;
}

NS_IMETHODIMP PromptService::Confirm(nsIDOMWindow* aParent,
                                     const PRUnichar* aDialogTitle,
                                     const PRUnichar* aDialogText, PRBool* aConfirm)
{
    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Confirm").get());
    prompter.SetMessageText(aDialogText);
    prompter.Create(MozillaPrompter::TYPE_CONFIRM, GetWindowForDOMWindow(aParent));
    prompter.Run();
    prompter.GetConfirmValue(aConfirm);
    return NS_OK;
}

NS_IMETHODIMP PromptService::ConfirmCheck(nsIDOMWindow* aParent,
                                          const PRUnichar* aDialogTitle,
                                          const PRUnichar* aDialogText,
                                          const PRUnichar* aCheckMsg,
                                          PRBool* aCheckValue, PRBool* aConfirm)
{
    NS_ENSURE_ARG_POINTER(aCheckValue);

    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Confirm").get());
    prompter.SetMessageText(aDialogText);
    prompter.SetCheckMessage(aCheckMsg);
    prompter.SetCheckValue(*aCheckValue);
    prompter.Create(MozillaPrompter::TYPE_CONFIRM_CHECK, GetWindowForDOMWindow(aParent));
    prompter.Run();
    prompter.GetCheckValue(aCheckValue);
    prompter.GetConfirmValue(aConfirm);
    return NS_OK;
}

NS_IMETHODIMP PromptService::ConfirmEx(nsIDOMWindow* aParent,
                                        const PRUnichar* aDialogTitle,
                                        const PRUnichar* aDialogText,
                                        PRUint32 aButtonFlags,
                                        const PRUnichar* aButton0Title,
                                        const PRUnichar* aButton1Title,
                                        const PRUnichar* aButton2Title,
                                        const PRUnichar* aCheckMsg, PRBool* aCheckValue,
                                        PRInt32* aRetVal)
{
    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Confirm").get());
    prompter.SetMessageText(aDialogText);

    nsAutoString button0Label, button1Label, button2Label;
    GetButtonLabel(aButtonFlags, BUTTON_POS_0, aButton0Title, button0Label);
    GetButtonLabel(aButtonFlags, BUTTON_POS_1, aButton1Title, button1Label);
    GetButtonLabel(aButtonFlags, BUTTON_POS_2, aButton2Title, button2Label);
    prompter.SetButtons(button0Label.get(), button1Label.get(), button2Label.get());

    if (aCheckMsg)
        prompter.SetCheckMessage(aCheckMsg);
    if (aCheckValue)
        prompter.SetCheckValue(*aCheckValue);

    prompter.Create(MozillaPrompter::TYPE_UNIVERSAL, GetWindowForDOMWindow(aParent));
    prompter.Run();

    if (aCheckValue)
        prompter.GetCheckValue(aCheckValue);

    prompter.GetButtonPressed(aRetVal);
    return NS_OK;
}

NS_IMETHODIMP PromptService::Prompt(nsIDOMWindow* aParent, const PRUnichar* aDialogTitle,
                                    const PRUnichar* aDialogText, PRUnichar** aValue,
                                    const PRUnichar* aCheckMsg, PRBool* aCheckValue,
                                    PRBool* aConfirm)
{
    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Prompt").get());
    prompter.SetMessageText(aDialogText);
    prompter.SetTextValue(*aValue);
    if (aCheckMsg)
        prompter.SetCheckMessage(aCheckMsg);
    if (aCheckValue)
        prompter.SetCheckValue(*aCheckValue);

    prompter.Create(MozillaPrompter::TYPE_PROMPT, GetWindowForDOMWindow(aParent));
    prompter.Run();
    if (aCheckValue)
        prompter.GetCheckValue(aCheckValue);
    prompter.GetConfirmValue(aConfirm);
    if (*aConfirm) {
        if (*aValue)
            nsMemory::Free(*aValue);
        prompter.GetTextValue(aValue);
    }
    return NS_OK;
}

NS_IMETHODIMP PromptService::PromptUsernameAndPassword(nsIDOMWindow* aParent,
                                                        const PRUnichar* aDialogTitle,
                                                        const PRUnichar* aDialogText,
                                                        PRUnichar** aUsername,
                                                        PRUnichar** aPassword,
                                                        const PRUnichar* aCheckMsg,
                                                        PRBool* aCheckValue,
                                                        PRBool* aConfirm)
{
    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Prompt").get());
    prompter.SetMessageText(aDialogText);
    prompter.SetUser(*aUsername);
    prompter.SetPassword(*aPassword);
    if (aCheckMsg)
        prompter.SetCheckMessage(aCheckMsg);
    if (aCheckValue)
        prompter.SetCheckValue(*aCheckValue);

    prompter.Create(MozillaPrompter::TYPE_PROMPT_USER_PASS, GetWindowForDOMWindow(aParent));
    prompter.Run();
    if (aCheckValue)
        prompter.GetCheckValue(aCheckValue);
    prompter.GetConfirmValue(aConfirm);
    if (*aConfirm) {
        if (*aUsername)
            nsMemory::Free(*aUsername);
        prompter.GetUser(aUsername);

        if (*aPassword)
            nsMemory::Free(*aPassword);
        prompter.GetPassword(aPassword);
    }
    return NS_OK;
}

NS_IMETHODIMP PromptService::PromptPassword(nsIDOMWindow* aParent,
                                            const PRUnichar* aDialogTitle,
                                            const PRUnichar* aDialogText,
                                            PRUnichar** aPassword,
                                            const PRUnichar* aCheckMsg,
                                            PRBool* aCheckValue, PRBool* aConfirm)
{
    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Prompt").get());
    prompter.SetMessageText(aDialogText);
    prompter.SetPassword(*aPassword);
    if (aCheckMsg)
        prompter.SetCheckMessage(aCheckMsg);
    if (aCheckValue)
        prompter.SetCheckValue(*aCheckValue);

    prompter.Create(MozillaPrompter::TYPE_PROMPT_PASS, GetWindowForDOMWindow(aParent));
    prompter.Run();
    if (aCheckValue)
        prompter.GetCheckValue(aCheckValue);
    prompter.GetConfirmValue(aConfirm);
    if (*aConfirm) {
        if (*aPassword)
            nsMemory::Free(*aPassword);
        prompter.GetPassword(aPassword);
    }
    return NS_OK;
}

NS_IMETHODIMP PromptService::Select(nsIDOMWindow* aParent, const PRUnichar* aDialogTitle,
                                    const PRUnichar* aDialogText, PRUint32 aCount,
                                    const PRUnichar** aSelectList, PRInt32* outSelection,
                                    PRBool* aConfirm)
{
    MozillaPrompter prompter;
    prompter.SetTitle(aDialogTitle ? aDialogTitle : NS_LITERAL_STRING("Select").get());
    prompter.SetMessageText(aDialogText);
    prompter.SetItems(aSelectList, aCount);
    prompter.Create(MozillaPrompter::TYPE_SELECT, GetWindowForDOMWindow(aParent));
    prompter.Run();
    prompter.GetSelectedItem(outSelection);
    prompter.GetConfirmValue(aConfirm);
    return NS_OK;
}

GtkWindow* PromptService::GetWindowForDOMWindow(nsIDOMWindow* aDOMWindow)
{
    nsCOMPtr<nsIWindowWatcher> wwatch = do_GetService("@mozilla.org/embedcomp/window-watcher;1");

    if (!aDOMWindow)
        return NULL;

    nsCOMPtr<nsIWebBrowserChrome> chrome;
    wwatch->GetChromeForWindow(aDOMWindow, getter_AddRefs(chrome));
    nsCOMPtr<nsIEmbeddingSiteWindow> siteWindow = do_QueryInterface(chrome);

    if (!siteWindow)
        return NULL;

    GtkWidget* parentWidget;
    siteWindow->GetSiteWindow((void**)&parentWidget);

    if (!parentWidget)
        return NULL;

    GtkWidget* gtkWin = gtk_widget_get_toplevel(parentWidget);
    if (GTK_WIDGET_TOPLEVEL(gtkWin))
        return GTK_WINDOW(gtkWin);

    return NULL;
}

void PromptService::GetButtonLabel(PRUint32 aFlags, PRUint32 aPos,
                                    const PRUnichar* aStringValue,
                                    nsAString& aLabel)
{
    PRUint32 posFlag = (aFlags & (255 * aPos)) / aPos;
    switch (posFlag) {
    case 0:
        break;
    case BUTTON_TITLE_OK:
        aLabel.AssignLiteral(GTK_STOCK_OK);
        break;
    case BUTTON_TITLE_CANCEL:
        aLabel.AssignLiteral(GTK_STOCK_CANCEL);
        break;
    case BUTTON_TITLE_YES:
        aLabel.AssignLiteral(GTK_STOCK_YES);
        break;
    case BUTTON_TITLE_NO:
        aLabel.AssignLiteral(GTK_STOCK_NO);
        break;
    case BUTTON_TITLE_SAVE:
        aLabel.AssignLiteral(GTK_STOCK_SAVE);
        break;
    case BUTTON_TITLE_DONT_SAVE:
        aLabel.AssignLiteral("Don't Save");
        break;
    case BUTTON_TITLE_REVERT:
        aLabel.AssignLiteral("Revert");
        break;
    case BUTTON_TITLE_IS_STRING:
        aLabel = aStringValue;
        break;
    default:
        NS_WARNING("Unexpected button flags");
    }
}
